"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const serum_1 = require("@project-serum/serum");
const fixednum_1 = require("./fixednum");
const layout_1 = require("./layout");
const utils_1 = require("./utils");
const MangoGroup_1 = require("./MangoGroup");
class MangoAccount {
    constructor(publicKey, decoded) {
        this.publicKey = publicKey;
        this.spotOpenOrdersAccounts = new Array(layout_1.MAX_PAIRS).fill(undefined);
        Object.assign(this, decoded);
    }
    reload(connection) {
        return __awaiter(this, void 0, void 0, function* () {
            const acc = yield connection.getAccountInfo(this.publicKey);
            Object.assign(this, layout_1.MangoAccountLayout.decode(acc === null || acc === void 0 ? void 0 : acc.data));
            return this;
        });
    }
    getNativeDeposit(rootBank, tokenIndex) {
        return rootBank.depositIndex.mul(this.deposits[tokenIndex]);
    }
    getNativeBorrow(rootBank, tokenIndex) {
        return rootBank.borrowIndex.mul(this.borrows[tokenIndex]);
    }
    getUiDeposit(rootBank, mangoGroup, tokenIndex) {
        return this.getNativeDeposit(rootBank, tokenIndex).div(fixednum_1.I80F48.fromNumber(Math.pow(10, mangoGroup.tokens[tokenIndex].decimals)));
    }
    getUiBorrow(rootBank, mangoGroup, tokenIndex) {
        return this.getNativeBorrow(rootBank, tokenIndex).div(fixednum_1.I80F48.fromNumber(Math.pow(10, mangoGroup.tokens[tokenIndex].decimals)));
    }
    // TODO: use getMultipleAccounts instead
    loadOpenOrders(connection, serumDexPk) {
        return __awaiter(this, void 0, void 0, function* () {
            const promises = [];
            for (let i = 0; i < this.spotOpenOrders.length; i++) {
                if (this.spotOpenOrders[i].equals(utils_1.zeroKey)) {
                    promises.push(utils_1.promiseUndef());
                }
                else {
                    promises.push(serum_1.OpenOrders.load(connection, this.spotOpenOrders[i], serumDexPk));
                }
            }
            this.spotOpenOrdersAccounts = yield Promise.all(promises);
            return this.spotOpenOrdersAccounts;
        });
    }
    getSpotHealth(mangoCache, marketIndex, assetWeight, liabWeight) {
        const bankCache = mangoCache.rootBankCache[marketIndex];
        const price = mangoCache.priceCache[marketIndex].price;
        let [ooBase, ooQuote] = [fixednum_1.I80F48.fromString('0'), fixednum_1.I80F48.fromString('0')];
        const oo = this.spotOpenOrdersAccounts[marketIndex];
        if (oo !== undefined) {
            [ooBase, ooQuote] = [
                fixednum_1.I80F48.fromU64(oo.baseTokenTotal),
                fixednum_1.I80F48.fromU64(oo.quoteTokenTotal.add(oo['referrerRebatesAccrued'])),
            ];
        }
        const baseAssets = this.getNativeDeposit(bankCache, marketIndex).add(ooBase);
        const baseLiabs = this.getNativeBorrow(bankCache, marketIndex);
        // health = (baseAssets * aWeight - baseLiabs * lWeight) * price + ooQuote
        return baseAssets
            .mul(assetWeight)
            .sub(baseLiabs.mul(liabWeight))
            .mul(price)
            .add(ooQuote);
    }
    getHealth(mangoGroup, mangoCache, healthType) {
        const quoteDeposits = this.getNativeDeposit(mangoCache.rootBankCache[MangoGroup_1.QUOTE_INDEX], MangoGroup_1.QUOTE_INDEX);
        const quoteBorrows = this.getNativeBorrow(mangoCache.rootBankCache[MangoGroup_1.QUOTE_INDEX], MangoGroup_1.QUOTE_INDEX);
        let health = quoteDeposits.sub(quoteBorrows);
        for (let i = 0; i < mangoGroup.numOracles; i++) {
            const spotMarket = mangoGroup.spotMarkets[i];
            const perpMarket = mangoGroup.perpMarkets[i];
            const [spotAssetWeight, spotLiabWeight, perpAssetWeight, perpLiabWeight] = healthType === 'Maint'
                ? [
                    spotMarket.maintAssetWeight,
                    spotMarket.maintLiabWeight,
                    perpMarket.maintAssetWeight,
                    perpMarket.maintLiabWeight,
                ]
                : [
                    spotMarket.initAssetWeight,
                    spotMarket.initLiabWeight,
                    perpMarket.initAssetWeight,
                    perpMarket.initLiabWeight,
                ];
            if (!mangoGroup.spotMarkets[i].isEmpty()) {
                health = health.add(this.getSpotHealth(mangoCache, i, spotAssetWeight, spotLiabWeight));
            }
            if (!mangoGroup.perpMarkets[i].isEmpty()) {
                const perpsCache = mangoCache.perpMarketCache[i];
                health = health.add(this.perpAccounts[i].getHealth(perpMarket, mangoCache.priceCache[i].price, perpAssetWeight, perpLiabWeight, perpsCache.longFunding, perpsCache.shortFunding));
            }
        }
        return health;
    }
}
exports.default = MangoAccount;
//# sourceMappingURL=MangoAccount.js.map