import { Connection, PublicKey } from '@solana/web3.js';
import BN from 'bn.js';
import { I80F48 } from './fixednum';
import { MetaData, TokenInfo, SpotMarketInfo, PerpMarketInfo, MangoCache } from './layout';
import PerpMarket from './PerpMarket';
import RootBank from './RootBank';
export declare const MAX_TOKENS = 32;
export declare const MAX_PAIRS: number;
export declare const MAX_NODE_BANKS = 8;
export declare const QUOTE_INDEX: number;
export default class MangoGroup {
    publicKey: PublicKey;
    metaData: MetaData;
    numOracles: number;
    tokens: TokenInfo[];
    spotMarkets: SpotMarketInfo[];
    perpMarkets: PerpMarketInfo[];
    oracles: PublicKey[];
    signerNonce: BN;
    signerKey: PublicKey;
    admin: PublicKey;
    dexProgramId: PublicKey;
    mangoCache: PublicKey;
    validInterval: number[];
    rootBankAccounts: (RootBank | undefined)[];
    constructor(publicKey: PublicKey, decoded: any);
    getOracleIndex(oracle: PublicKey): number;
    getSpotMarketIndex(spotMarket: PublicKey): number;
    getPerpMarketIndex(perpMarket: PerpMarket): number;
    getTokenIndex(token: PublicKey): number;
    getRootBankIndex(rootBank: PublicKey): number;
    getBorrowRate(tokenIndex: number): I80F48;
    getDepositRate(tokenIndex: number): I80F48;
    loadCache(connection: Connection): Promise<MangoCache>;
    loadRootBanks(connection: Connection): Promise<(RootBank | undefined)[]>;
    loadPerpMarket(connection: Connection, marketIndex: number, baseDecimals: number, quoteDecimals: number): Promise<PerpMarket>;
}
//# sourceMappingURL=MangoGroup.d.ts.map