"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.QUOTE_INDEX = exports.MAX_NODE_BANKS = exports.MAX_PAIRS = exports.MAX_TOKENS = void 0;
const layout_1 = require("./layout");
const PerpMarket_1 = __importDefault(require("./PerpMarket"));
const RootBank_1 = __importDefault(require("./RootBank"));
const utils_1 = require("./utils");
exports.MAX_TOKENS = 32;
exports.MAX_PAIRS = exports.MAX_TOKENS - 1;
exports.MAX_NODE_BANKS = 8;
exports.QUOTE_INDEX = exports.MAX_TOKENS - 1;
class MangoGroup {
    constructor(publicKey, decoded) {
        this.publicKey = publicKey;
        Object.assign(this, decoded);
        this.oracles = this.oracles.filter((o) => !o.equals(utils_1.zeroKey));
        this.rootBankAccounts = new Array(exports.MAX_TOKENS).fill(undefined);
    }
    getOracleIndex(oracle) {
        for (let i = 0; i < this.numOracles; i++) {
            if (this.oracles[i].equals(oracle)) {
                return i;
            }
        }
        throw new Error('This Oracle does not belong to this MangoGroup');
    }
    getSpotMarketIndex(spotMarket) {
        for (let i = 0; i < this.numOracles; i++) {
            if (this.spotMarkets[i].spotMarket.equals(spotMarket)) {
                return i;
            }
        }
        throw new Error('This Market does not belong to this MangoGroup');
    }
    getPerpMarketIndex(perpMarket) {
        for (let i = 0; i < this.numOracles; i++) {
            if (this.perpMarkets[i].perpMarket.equals(perpMarket.publicKey)) {
                return i;
            }
        }
        throw new Error('This PerpMarket does not belong to this MangoGroup');
    }
    getTokenIndex(token) {
        for (let i = 0; i < this.tokens.length; i++) {
            if (this.tokens[i].mint.equals(token)) {
                return i;
            }
        }
        throw new Error('This token does not belong in this MangoGroup');
    }
    getRootBankIndex(rootBank) {
        for (let i = 0; i < this.tokens.length; i++) {
            if (this.tokens[i].rootBank.equals(rootBank)) {
                return i;
            }
        }
        throw new Error('This root bank does not belong in this MangoGroup');
    }
    getBorrowRate(tokenIndex) {
        const rootBank = this.rootBankAccounts[tokenIndex];
        if (!rootBank)
            throw new Error(`Root bank at index ${tokenIndex} is not loaded`);
        return rootBank.getBorrowRate(this);
    }
    getDepositRate(tokenIndex) {
        const rootBank = this.rootBankAccounts[tokenIndex];
        if (!rootBank)
            throw new Error(`Root bank at index ${tokenIndex} is not loaded`);
        return rootBank.getDepositRate(this);
    }
    loadCache(connection) {
        return __awaiter(this, void 0, void 0, function* () {
            const account = yield connection.getAccountInfo(this.mangoCache);
            if (!account || !(account === null || account === void 0 ? void 0 : account.data))
                throw new Error('Unable to load cache');
            const decoded = layout_1.MangoCacheLayout.decode(account.data);
            return new layout_1.MangoCache(this.mangoCache, decoded);
        });
    }
    loadRootBanks(connection) {
        return __awaiter(this, void 0, void 0, function* () {
            const promises = [];
            for (let i = 0; i < this.tokens.length; i++) {
                if (this.tokens[i].rootBank.equals(utils_1.zeroKey)) {
                    promises.push(utils_1.promiseUndef());
                }
                else {
                    promises.push(connection.getAccountInfo(this.tokens[i].rootBank));
                }
            }
            const accounts = yield Promise.all(promises);
            const parsedRootBanks = accounts.map((acc, i) => {
                if (acc && acc.data) {
                    const decoded = layout_1.RootBankLayout.decode(acc.data);
                    return new RootBank_1.default(this.tokens[i].rootBank, decoded);
                }
                return undefined;
            });
            yield Promise.all(parsedRootBanks.map((rootBank) => rootBank ? rootBank.loadNodeBanks(connection) : utils_1.promiseUndef()));
            this.rootBankAccounts = parsedRootBanks;
            return parsedRootBanks;
        });
    }
    loadPerpMarket(connection, marketIndex, baseDecimals, quoteDecimals) {
        return __awaiter(this, void 0, void 0, function* () {
            const pk = this.perpMarkets[marketIndex].perpMarket;
            const acc = yield connection.getAccountInfo(pk);
            const decoded = layout_1.PerpMarketLayout.decode(acc === null || acc === void 0 ? void 0 : acc.data);
            return new PerpMarket_1.default(pk, baseDecimals, quoteDecimals, decoded);
        });
    }
}
exports.default = MangoGroup;
//# sourceMappingURL=MangoGroup.js.map