#!/usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeConfig = exports.readConfig = void 0;
const fs = __importStar(require("fs"));
const os = __importStar(require("os"));
const yargs_1 = __importDefault(require("yargs/yargs"));
const helpers_1 = require("yargs/helpers");
const web3_js_1 = require("@solana/web3.js");
const commands_1 = require("./commands");
const config_1 = require("./config");
const clusterDesc = [
    'cluster',
    {
        describe: 'the cluster to connect to',
        default: 'devnet',
        choices: ['devnet', 'mainnet'],
    },
];
const configDesc = [
    'config',
    {
        describe: 'the config file to store all public keys',
        default: __dirname + '/ids.json',
        type: 'string',
    },
];
const keypairDesc = [
    'keypair',
    {
        describe: 'the keypair used to sign all transactions',
        default: os.homedir() + '/.config/solana/devnet.json',
        type: 'string',
    },
];
const groupDesc = [
    'group',
    { describe: 'the mango group name 🥭', type: 'string' },
];
const symbolDesc = [
    'symbol',
    { describe: 'the base token symbol', type: 'string' },
];
function openConnection(config, cluster) {
    return new web3_js_1.Connection(config.cluster_urls[cluster], 'processed');
}
function readKeypair(keypairPath) {
    return new web3_js_1.Account(JSON.parse(fs.readFileSync(keypairPath, 'utf-8')));
}
function readConfig(configPath) {
    return new config_1.Config(JSON.parse(fs.readFileSync(configPath, 'utf-8')));
}
exports.readConfig = readConfig;
function writeConfig(configPath, config) {
    fs.writeFileSync(configPath, JSON.stringify(config.toJson(), null, 2));
}
exports.writeConfig = writeConfig;
//@ts-ignore
yargs_1.default(helpers_1.hideBin(process.argv))
    .command('init-group <group> <mangoProgramId> <serumProgramId> <quote_mint>', 'initialize a new group', (y) => y
    .positional(...groupDesc)
    .positional('mangoProgramId', {
    describe: 'the program id of the mango smart contract',
    type: 'string',
})
    .positional('serumProgramId', {
    describe: 'the program id of the serum dex smart contract',
    type: 'string',
})
    .positional('quote_mint', {
    describe: 'the mint of the quote currency 💵',
    type: 'string',
})
    .option('symbol', {
    describe: 'the quote symbol',
    default: 'USDC',
    type: 'string',
})
    .option(...clusterDesc)
    .option(...configDesc)
    .option(...keypairDesc), (args) => __awaiter(void 0, void 0, void 0, function* () {
    console.log('init_group', args);
    const mangoProgramId = new web3_js_1.PublicKey(args.mangoProgramId);
    const serumProgramId = new web3_js_1.PublicKey(args.serumProgramId);
    const quoteMint = new web3_js_1.PublicKey(args.quote_mint);
    const account = readKeypair(args.keypair);
    const config = readConfig(args.config);
    const cluster = args.cluster;
    const connection = openConnection(config, cluster);
    const result = yield commands_1.initGroup(connection, account, cluster, args.group, mangoProgramId, serumProgramId, args.symbol, quoteMint);
    config.storeGroup(result);
    writeConfig(args.config, config);
    process.exit(0);
}))
    .command('add-oracle <group> <symbol>', 'add an oracle to the group', (y) => y
    .positional(...groupDesc)
    .positional(...symbolDesc)
    .option('provider', {
    describe: 'oracle provider',
    default: 'stub',
    choices: ['stub' /*, 'pyth'*/],
})
    .option(...clusterDesc)
    .option(...configDesc)
    .option(...keypairDesc), (args) => __awaiter(void 0, void 0, void 0, function* () {
    console.log('add_oracle', args);
    const account = readKeypair(args.keypair);
    const config = readConfig(args.config);
    const cluster = args.cluster;
    const connection = openConnection(config, cluster);
    const group = config.getGroup(cluster, args.group);
    const result = yield commands_1.addStubOracle(connection, account, group, args.symbol);
    config.storeGroup(result);
    writeConfig(args.config, config);
    process.exit(0);
}))
    .command('set-oracle <group> <symbol> <value>', 'set stub oracle to given value', (y) => y
    .positional(...groupDesc)
    .positional(...symbolDesc)
    .positional('value', {
    describe: 'new oracle value is base_price * quote_unit / base_unit',
    type: 'number',
})
    .option(...clusterDesc)
    .option(...configDesc)
    .option(...keypairDesc), (args) => __awaiter(void 0, void 0, void 0, function* () {
    console.log('set_oracle', args);
    const account = readKeypair(args.keypair);
    const config = readConfig(args.config);
    const cluster = args.cluster;
    const connection = openConnection(config, cluster);
    const group = config.getGroup(cluster, args.group);
    yield commands_1.setStubOracle(connection, account, group, args.symbol, args.value);
    process.exit(0);
}))
    .command('add-perp-market <group> <symbol>', 'add a perp market to the group', (y) => y
    .positional(...groupDesc)
    .positional(...symbolDesc)
    .option('maint_leverage', {
    default: 20,
    type: 'number',
})
    .option('init_leverage', {
    default: 10,
    type: 'number',
})
    .option('base_lot_size', {
    default: 100,
    type: 'number',
})
    .option('quote_lot_size', {
    default: 10,
    type: 'number',
})
    .option('max_num_events', {
    default: 128,
    type: 'number',
})
    .option(...clusterDesc)
    .option(...configDesc)
    .option(...keypairDesc), (args) => __awaiter(void 0, void 0, void 0, function* () {
    console.log('add-perp-market', args);
    const account = readKeypair(args.keypair);
    const config = readConfig(args.config);
    const cluster = args.cluster;
    const connection = openConnection(config, cluster);
    const group = config.getGroup(cluster, args.group);
    const result = yield commands_1.addPerpMarket(connection, account, group, args.symbol, args.maint_leverage, args.init_leverage, args.base_lot_size, args.quote_lot_size, args.max_num_events);
    config.storeGroup(result);
    writeConfig(args.config, config);
    process.exit(0);
}))
    .command('add-spot-market <group> <symbol> <market_pk> <mint_pk>', 'add a perp market to the group', (y) => y
    .positional(...groupDesc)
    .positional(...symbolDesc)
    .positional('market_pk', {
    describe: 'the public key of the spot market',
    type: 'string',
})
    .positional('mint_pk', {
    describe: 'the public key of the base token mint',
    type: 'string',
})
    .option('maint_leverage', {
    default: 10,
    type: 'number',
})
    .option('init_leverage', {
    default: 5,
    type: 'number',
})
    .option(...clusterDesc)
    .option(...configDesc)
    .option(...keypairDesc), (args) => __awaiter(void 0, void 0, void 0, function* () {
    console.log('add-spot-market', args);
    const account = readKeypair(args.keypair);
    const config = readConfig(args.config);
    const cluster = args.cluster;
    const connection = openConnection(config, cluster);
    const group = config.getGroup(cluster, args.group);
    const result = yield commands_1.addSpotMarket(connection, account, group, args.symbol, new web3_js_1.PublicKey(args.market_pk), new web3_js_1.PublicKey(args.mint_pk), args.maint_leverage, args.init_leverage);
    config.storeGroup(result);
    writeConfig(args.config, config);
    process.exit(0);
})).argv;
//# sourceMappingURL=cli.js.map