import { Account, Connection, PublicKey, Transaction, TransactionConfirmationStatus, TransactionSignature } from '@solana/web3.js';
import BN from 'bn.js';
import { EventQueue } from './layout';
import MangoGroup from './MangoGroup';
import MangoAccount from './MangoAccount';
import PerpMarket from './PerpMarket';
import RootBank from './RootBank';
import { Market } from '@project-serum/serum';
import { I80F48 } from './fixednum';
import { Order } from '@project-serum/serum/lib/market';
import { WalletAdapter } from './types';
import { PerpOrder } from './book';
export declare const getUnixTs: () => number;
export declare class MangoClient {
    connection: Connection;
    programId: PublicKey;
    constructor(connection: Connection, programId: PublicKey);
    sendTransactions(transactions: Transaction[], payer: Account | WalletAdapter, additionalSigners: Account[], timeout?: number, confirmLevel?: TransactionConfirmationStatus): Promise<TransactionSignature[]>;
    signTransaction({ transaction, payer, signers }: {
        transaction: any;
        payer: any;
        signers: any;
    }): Promise<any>;
    sendTransaction(transaction: Transaction, payer: Account | WalletAdapter, additionalSigners: Account[], timeout?: number, confirmLevel?: TransactionConfirmationStatus): Promise<TransactionSignature>;
    initMangoGroup(quoteMint: PublicKey, dexProgram: PublicKey, validInterval: number, payer: Account | WalletAdapter): Promise<PublicKey>;
    getMangoGroup(mangoGroup: PublicKey): Promise<MangoGroup>;
    initMangoAccount(mangoGroup: MangoGroup, owner: Account | WalletAdapter): Promise<PublicKey>;
    getMangoAccount(mangoAccountPk: PublicKey, dexProgramId: PublicKey): Promise<MangoAccount>;
    deposit(mangoGroup: MangoGroup, mangoAccount: MangoAccount, owner: Account | WalletAdapter, rootBank: PublicKey, nodeBank: PublicKey, vault: PublicKey, tokenAcc: PublicKey, quantity: number): Promise<TransactionSignature>;
    withdraw(mangoGroup: MangoGroup, mangoAccount: MangoAccount, owner: Account | WalletAdapter, rootBank: PublicKey, nodeBank: PublicKey, vault: PublicKey, tokenAcc: PublicKey, quantity: number, allowBorrow: boolean): Promise<TransactionSignature>;
    cacheRootBanks(mangoGroup: PublicKey, mangoCache: PublicKey, rootBanks: PublicKey[], payer: Account | WalletAdapter): Promise<TransactionSignature>;
    cachePrices(mangoGroup: PublicKey, mangoCache: PublicKey, oracles: PublicKey[], payer: Account | WalletAdapter): Promise<TransactionSignature>;
    cachePerpMarkets(mangoGroup: PublicKey, mangoCache: PublicKey, perpMarkets: PublicKey[], payer: Account): Promise<TransactionSignature>;
    updateRootBank(mangoGroup: PublicKey, rootBank: PublicKey, nodeBanks: PublicKey[], payer: Account | WalletAdapter): Promise<TransactionSignature>;
    consumeEvents(mangoGroup: PublicKey, perpMarket: PublicKey, eventQueue: PublicKey, mangoAccounts: PublicKey[], payer: Account, limit: BN): Promise<TransactionSignature>;
    updateFunding(mangoGroup: PublicKey, mangoCache: PublicKey, perpMarket: PublicKey, bids: PublicKey, asks: PublicKey, payer: Account): Promise<TransactionSignature>;
    getPerpMarket(perpMarketPk: PublicKey, baseDecimal: number, quoteDecimal: number): Promise<PerpMarket>;
    placePerpOrder(mangoGroup: MangoGroup, mangoAccount: MangoAccount, mangoCache: PublicKey, perpMarket: PerpMarket, owner: Account | WalletAdapter, side: 'buy' | 'sell', price: number, quantity: number, orderType?: 'limit' | 'ioc' | 'postOnly', clientOrderId?: number): Promise<TransactionSignature>;
    cancelPerpOrder(mangoGroup: MangoGroup, mangoAccount: MangoAccount, owner: Account | WalletAdapter, perpMarket: PerpMarket, order: PerpOrder): Promise<TransactionSignature>;
    addOracle(mangoGroup: MangoGroup, oracle: PublicKey, admin: Account): Promise<TransactionSignature>;
    setOracle(mangoGroup: MangoGroup, oracle: PublicKey, admin: Account, price: I80F48): Promise<TransactionSignature>;
    addSpotMarket(mangoGroup: MangoGroup, spotMarket: PublicKey, mint: PublicKey, admin: Account, marketIndex: number, maintLeverage: number, initLeverage: number): Promise<TransactionSignature>;
    placeSpotOrder(mangoGroup: MangoGroup, mangoAccount: MangoAccount, mangoCache: PublicKey, spotMarket: Market, owner: Account | WalletAdapter, side: 'buy' | 'sell', price: number, size: number, orderType?: 'limit' | 'ioc' | 'postOnly'): Promise<TransactionSignature>;
    cancelSpotOrder(mangoGroup: MangoGroup, mangoAccount: MangoAccount, owner: Account | WalletAdapter, spotMarket: Market, order: Order): Promise<TransactionSignature>;
    settleFunds(mangoGroup: MangoGroup, mangoAccount: MangoAccount, owner: Account | WalletAdapter, spotMarket: Market): Promise<TransactionSignature>;
    /**
     * Automatically fetch MangoAccounts for this PerpMarket
     * Pick enough MangoAccounts that have opposite sign and send them in to get settled
     */
    settlePnl(mangoGroup: MangoGroup, mangoAccount: MangoAccount, perpMarket: PerpMarket, quoteRootBank: RootBank, price: I80F48, // should be the MangoCache price
    owner: Account | WalletAdapter): Promise<TransactionSignature | null>;
    getMarginAccountsForOwner(mangoGroup: MangoGroup, owner: PublicKey, includeOpenOrders?: boolean): Promise<MangoAccount[]>;
    getAllMangoAccounts(mangoGroup: MangoGroup, filters?: any[], includeOpenOrders?: boolean): Promise<MangoAccount[]>;
    addStubOracle(mangoGroupPk: PublicKey, admin: Account): Promise<string>;
    setStubOracle(mangoGroupPk: PublicKey, oraclePk: PublicKey, admin: Account, price: number): Promise<string>;
    addPerpMarket(mangoGroupPk: PublicKey, admin: Account, marketIndex: number, maintLeverage: number, initLeverage: number, baseLotSize: number, quoteLotSize: number, maxNumEvents: number): Promise<string>;
    getOrderBook(): Promise<void>;
    getEventQueue(eventQueue: PublicKey): Promise<EventQueue>;
}
//# sourceMappingURL=client.d.ts.map