import { PublicKey, TransactionInstruction } from '@solana/web3.js';
import BN from 'bn.js';
import { Order } from '@project-serum/serum/lib/market';
import { I80F48 } from './fixednum';
import { PerpOrder } from '.';
export declare function makeInitMangoGroupInstruction(programId: PublicKey, mangoGroupPk: PublicKey, signerKey: PublicKey, payerPk: PublicKey, quoteMintPk: PublicKey, quoteVaultPk: PublicKey, quoteNodeBankPk: PublicKey, quoteRootBankPk: PublicKey, mangoCachePk: PublicKey, dexProgramPk: PublicKey, signerNonce: BN, validInterval: BN): TransactionInstruction;
export declare function makeInitMangoAccountInstruction(programId: PublicKey, mangoGroupPk: PublicKey, mangoAccountPk: PublicKey, ownerPk: PublicKey): TransactionInstruction;
export declare function makeTestMultiTxInstruction(programId: PublicKey, mangoGroup: PublicKey, index: BN): TransactionInstruction;
export declare function makeWithdrawInstruction(programId: PublicKey, mangoGroupPk: PublicKey, mangoAccountPk: PublicKey, walletPk: PublicKey, mangoCachePk: PublicKey, rootBankPk: PublicKey, nodeBankPk: PublicKey, vaultPk: PublicKey, tokenAccPk: PublicKey, signerKey: PublicKey, openOrders: PublicKey[], nativeQuantity: BN, allowBorrow: boolean): TransactionInstruction;
export declare function makeSettleFundsInstruction(programId: PublicKey, mangoGroupPk: PublicKey, ownerPk: PublicKey, mangoAccountPk: PublicKey, dexProgramId: PublicKey, spotMarketPk: PublicKey, openOrdersPk: PublicKey, signerKey: PublicKey, spotMarketBaseVaultPk: PublicKey, spotMarketQuoteVaultPk: PublicKey, baseRootBankPk: PublicKey, baseNodeBankPk: PublicKey, quoteRootBankPk: PublicKey, quoteNodeBankPk: PublicKey, baseVaultPk: PublicKey, quoteVaultPk: PublicKey, dexSignerKey: PublicKey): TransactionInstruction;
export declare function makeCancelSpotOrderInstruction(programId: PublicKey, mangoGroupPk: PublicKey, ownerPk: PublicKey, mangoAccountPk: PublicKey, dexProgramId: PublicKey, spotMarketPk: PublicKey, bidsPk: PublicKey, asksPk: PublicKey, openOrdersPk: PublicKey, signerKey: PublicKey, eventQueuePk: PublicKey, order: Order): TransactionInstruction;
export declare function makeCancelPerpOrderInstruction(programId: PublicKey, mangoGroupPk: PublicKey, mangoAccountPk: PublicKey, ownerPk: PublicKey, perpMarketPk: PublicKey, bidsPk: PublicKey, asksPk: PublicKey, eventQueuePk: PublicKey, order: PerpOrder): TransactionInstruction;
export declare function makeCancelPerpOrderByClientIdInstruction(programId: PublicKey, mangoGroupPk: PublicKey, mangoAccountPk: PublicKey, ownerPk: PublicKey, perpMarketPk: PublicKey, bidsPk: PublicKey, asksPk: PublicKey, eventQueuePk: PublicKey, clientOrderId: BN): TransactionInstruction;
export declare function makeDepositInstruction(programId: PublicKey, mangoGroupPk: PublicKey, ownerPk: PublicKey, merpsCachePk: PublicKey, mangoAccountPk: PublicKey, rootBankPk: PublicKey, nodeBankPk: PublicKey, vaultPk: PublicKey, tokenAccPk: PublicKey, nativeQuantity: BN): TransactionInstruction;
export declare function makeCacheRootBankInstruction(programId: PublicKey, mangoGroupPk: PublicKey, mangoCachePk: PublicKey, rootBanks: PublicKey[]): TransactionInstruction;
export declare function makeCachePricesInstruction(programId: PublicKey, mangoGroupPk: PublicKey, mangoCachePk: PublicKey, oracles: PublicKey[]): TransactionInstruction;
export declare function makeCachePerpMarketInstruction(programId: PublicKey, mangoGroupPk: PublicKey, mangoCachePk: PublicKey, perpMarketPks: PublicKey[]): TransactionInstruction;
export declare function makeAddSpotMarketInstruction(programId: PublicKey, mangoGroupPk: PublicKey, spotMarketPk: PublicKey, serumDexPk: PublicKey, mintPk: PublicKey, nodeBankPk: PublicKey, vaultPk: PublicKey, rootBankPk: PublicKey, adminPk: PublicKey, marketIndex: BN, maintLeverage: I80F48, initLeverage: I80F48): TransactionInstruction;
export declare function makePlaceSpotOrderInstruction(programId: PublicKey, mangoGroupPk: PublicKey, mangoAccountPk: PublicKey, ownerPk: PublicKey, mangoCachePk: PublicKey, serumDexPk: PublicKey, spotMarketPk: PublicKey, bidsPk: PublicKey, asksPk: PublicKey, requestQueuePk: PublicKey, eventQueuePk: PublicKey, spotMktBaseVaultPk: PublicKey, spotMktQuoteVaultPk: PublicKey, baseRootBankPk: PublicKey, baseNodeBankPk: PublicKey, quoteRootBankPk: PublicKey, quoteNodeBankPk: PublicKey, quoteVaultPk: PublicKey, baseVaultPk: PublicKey, signerPk: PublicKey, dexSignerPk: PublicKey, openOrders: PublicKey[], side: 'buy' | 'sell', limitPrice: BN, maxBaseQuantity: BN, maxQuoteQuantity: BN, selfTradeBehavior: string, orderType?: 'limit' | 'ioc' | 'postOnly'): TransactionInstruction;
export declare function makeUpdateRootBankInstruction(programId: PublicKey, mangoGroupPk: PublicKey, rootBankPk: PublicKey, nodeBanks: PublicKey[]): TransactionInstruction;
export declare function makeAddOracleInstruction(programId: PublicKey, mangoGroupPk: PublicKey, oraclePk: PublicKey, adminPk: PublicKey): TransactionInstruction;
export declare function makeSetOracleInstruction(programId: PublicKey, mangoGroupPk: PublicKey, oraclePk: PublicKey, adminPk: PublicKey, price: I80F48): TransactionInstruction;
export declare function makeAddPerpMarketInstruction(programId: PublicKey, mangoGroupPk: PublicKey, perpMarketPk: PublicKey, eventQueuePk: PublicKey, bidsPk: PublicKey, asksPk: PublicKey, adminPk: PublicKey, marketIndex: BN, maintLeverage: I80F48, initLeverage: I80F48, baseLotSize: BN, quoteLotSize: BN): TransactionInstruction;
export declare function makeCachePerpMarketsInstruction(programId: PublicKey, mangoGroupPk: PublicKey, mangoCachePk: PublicKey, perpMarkets: PublicKey[]): TransactionInstruction;
export declare function makeSettlePnlInstruction(programId: PublicKey, mangoGroupPk: PublicKey, mangoAccountAPk: PublicKey, mangoAccountBPk: PublicKey, mangoCachePk: PublicKey, rootBankPk: PublicKey, nodeBankPk: PublicKey, marketIndex: BN): TransactionInstruction;
export declare function makeConsumeEventsInstruction(programId: PublicKey, mangoGroupPk: PublicKey, perpMarketPk: PublicKey, eventQueuePk: PublicKey, mangoAccountPks: PublicKey[], limit: BN): TransactionInstruction;
export declare function makePlacePerpOrderInstruction(programId: PublicKey, mangoGroupPk: PublicKey, mangoAccountPk: PublicKey, ownerPk: PublicKey, mangoCachePk: PublicKey, perpMarketPk: PublicKey, bidsPk: PublicKey, asksPk: PublicKey, eventQueuePk: PublicKey, openOrders: PublicKey[], price: BN, quantity: BN, clientOrderId: BN, side: 'buy' | 'sell', orderType?: 'limit' | 'ioc' | 'postOnly'): TransactionInstruction;
export declare function makeUpdateFundingInstruction(programId: PublicKey, mangoGroupPk: PublicKey, mangoCachePk: PublicKey, perpMarketPk: PublicKey, bidsPk: PublicKey, asksPk: PublicKey): TransactionInstruction;
//# sourceMappingURL=instruction.d.ts.map