"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Keeper = void 0;
/**
This will probably move to its own repo at some point but easier to keep it here for now
This will be a long running program that will call all the Keeper related instructions on-chain

This will be very similar to the crank in serum dex.

 */
const os = __importStar(require("os"));
const fs = __importStar(require("fs"));
const client_1 = require("./client");
const web3_js_1 = require("@solana/web3.js");
const utils_1 = require("./utils");
const ids_json_1 = __importDefault(require("./ids.json"));
const config_1 = require("./config");
const MangoGroup_1 = require("../src/MangoGroup");
const instruction_1 = require("./instruction");
const bn_js_1 = __importDefault(require("bn.js"));
class Keeper {
    /**
     * Long running program that never exits except on keyboard interrupt
     */
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const interval = process.env.INTERVAL || 3500;
            const config = new config_1.Config(ids_json_1.default);
            const cluster = (process.env.CLUSTER || 'devnet');
            const groupName = process.env.GROUP || 'mango_test_v3.1';
            const groupIds = config.getGroup(cluster, groupName);
            if (!groupIds) {
                throw new Error(`Group ${groupName} not found`);
            }
            const mangoProgramId = groupIds.mangoProgramId;
            const mangoGroupKey = groupIds.publicKey;
            const payer = new web3_js_1.Account(JSON.parse(process.env.KEYPAIR ||
                fs.readFileSync(os.homedir() + '/.config/solana/devnet.json', 'utf-8')));
            const connection = new web3_js_1.Connection(config.cluster_urls[cluster], 'processed');
            const client = new client_1.MangoClient(connection, mangoProgramId);
            const mangoGroup = yield client.getMangoGroup(mangoGroupKey);
            const perpMarkets = yield Promise.all(groupIds.perpMarkets.map((m, i) => {
                return mangoGroup.loadPerpMarket(connection, i, m.baseDecimals, m.quoteDecimals);
            }));
            const lastSeqNums = {};
            perpMarkets.forEach((m) => {
                lastSeqNums[m.publicKey.toBase58()] = new bn_js_1.default(0);
            });
            // eslint-disable-next-line
            while (true) {
                yield utils_1.sleep(interval);
                const cacheTransaction = new web3_js_1.Transaction();
                cacheTransaction.add(instruction_1.makeCacheRootBankInstruction(mangoProgramId, mangoGroup.publicKey, mangoGroup.mangoCache, [
                    mangoGroup.tokens[0].rootBank,
                    mangoGroup.tokens[MangoGroup_1.QUOTE_INDEX].rootBank,
                ]));
                cacheTransaction.add(instruction_1.makeCachePricesInstruction(mangoProgramId, mangoGroup.publicKey, mangoGroup.mangoCache, mangoGroup.oracles));
                cacheTransaction.add(instruction_1.makeCachePerpMarketsInstruction(mangoProgramId, mangoGroup.publicKey, mangoGroup.mangoCache, mangoGroup.perpMarkets
                    .filter((pm) => !pm.isEmpty())
                    .map((pm) => pm.perpMarket)));
                const updateRootBankTransaction = new web3_js_1.Transaction();
                groupIds.tokens.forEach((token) => {
                    updateRootBankTransaction.add(instruction_1.makeUpdateRootBankInstruction(mangoProgramId, mangoGroup.publicKey, token.rootKey, token.nodeKeys));
                });
                const updateFundingTransaction = new web3_js_1.Transaction();
                perpMarkets.forEach((market) => {
                    if (market) {
                        updateFundingTransaction.add(instruction_1.makeUpdateFundingInstruction(mangoProgramId, mangoGroup.publicKey, mangoGroup.mangoCache, market.publicKey, market.bids, market.asks));
                    }
                });
                if (process.env.CONSUME_EVENTS == 'true') {
                    yield Promise.all(perpMarkets.map((m) => {
                        return m.loadEventQueue(connection).then((queue) => {
                            const accounts = [];
                            const events = queue.eventsSince(lastSeqNums[m.publicKey.toBase58()]);
                            events.forEach((ev) => {
                                if (ev.fill) {
                                    accounts.push(ev.fill.owner);
                                }
                                if (ev.out) {
                                    accounts.push(ev.out.owner);
                                }
                            });
                            client.consumeEvents(mangoGroup.publicKey, m.publicKey, m.eventQueue, [...new Map(accounts.map((a) => [a.toBase58(), a])).values()], payer, new bn_js_1.default(events.length));
                            console.log(`Consumed ${events.length} events`);
                            lastSeqNums[m.publicKey.toBase58()] = queue.seqNum;
                        });
                    }));
                }
                yield Promise.all([
                    client.sendTransaction(cacheTransaction, payer, []),
                    client.sendTransaction(updateRootBankTransaction, payer, []),
                    client.sendTransaction(updateFundingTransaction, payer, []),
                ]);
            }
        });
    }
}
exports.Keeper = Keeper;
new Keeper().run();
//# sourceMappingURL=keeper.js.map