/// <reference types="node" />
import { Blob, Structure, Layout, UInt } from 'buffer-layout';
import { PublicKey } from '@solana/web3.js';
import { I80F48 } from './fixednum';
import BN from 'bn.js';
export declare const MAX_TOKENS = 32;
export declare const MAX_PAIRS: number;
export declare const MAX_NODE_BANKS = 8;
export declare const MAX_RATE: I80F48;
export declare const OPTIMAL_UTIL: I80F48;
export declare const OPTIMAL_RATE: I80F48;
declare class _I80F48Layout extends Blob {
    constructor(property: string);
    decode(b: any, offset: any): I80F48;
    encode(src: any, b: any, offset: any): any;
}
export declare function I80F48Layout(property?: string): _I80F48Layout;
declare class BNLayout extends Blob {
    signed: boolean;
    constructor(number: number, property: any, signed?: boolean);
    decode(b: any, offset: any): BN;
    encode(src: any, b: any, offset: any): any;
}
export declare function u64(property?: string): BNLayout;
export declare function i64(property?: string): BNLayout;
export declare function u128(property?: string): BNLayout;
export declare function i128(property?: string): BNLayout;
declare class WrappedLayout<T, U> extends Layout<U> {
    layout: Layout<T>;
    decoder: (data: T) => U;
    encoder: (src: U) => T;
    constructor(layout: Layout<T>, decoder: (data: T) => U, encoder: (src: U) => T, property?: string);
    decode(b: Buffer, offset?: number): U;
    encode(src: U, b: Buffer, offset?: number): number;
    getSpan(b: Buffer, offset?: number): number;
}
export declare function bool(property?: string): WrappedLayout<number, boolean>;
declare class EnumLayout extends UInt {
    values: any;
    constructor(values: any, span: any, property: any);
    encode(src: any, b: any, offset: any): any;
    decode(b: any, offset: any): string;
}
export declare function sideLayout(property: any, span: any): EnumLayout;
export declare function orderTypeLayout(property: any, span: any): EnumLayout;
export declare function selfTradeBehaviorLayout(property: any): EnumLayout;
/**
 * Need to implement layouts for each of the structs found in state.rs
 */
export declare const MangoInstructionLayout: any;
export declare function encodeMangoInstruction(data: any): Buffer;
export declare class PublicKeyLayout extends Blob {
    constructor(property: any);
    decode(b: any, offset: any): PublicKey;
    encode(src: any, b: any, offset: any): any;
}
export declare function publicKeyLayout(property?: string): PublicKeyLayout;
export declare const DataType: {
    MangoGroup: number;
    MangoAccount: number;
    RootBank: number;
    NodeBank: number;
    PerpMarket: number;
    Bids: number;
    Asks: number;
    MangoCache: number;
    EventQueue: number;
};
export declare class MetaData {
    dataType: number;
    version: number;
    isInitialized: boolean;
    constructor(decoded: any);
}
export declare class MetaDataLayout extends Structure {
    constructor(property: any);
    decode(b: any, offset: any): MetaData;
    encode(src: any, b: any, offset: any): any;
}
export declare function metaDataLayout(property?: string): MetaDataLayout;
export declare class TokenInfo {
    mint: PublicKey;
    rootBank: PublicKey;
    decimals: number;
    padding: number[];
    constructor(decoded: any);
    isEmpty(): boolean;
}
export declare class TokenInfoLayout extends Structure {
    constructor(property: any);
    decode(b: any, offset: any): TokenInfo;
    encode(src: any, b: any, offset: any): any;
}
export declare function tokenInfoLayout(property?: string): TokenInfoLayout;
export declare class SpotMarketInfo {
    spotMarket: PublicKey;
    maintAssetWeight: I80F48;
    initAssetWeight: I80F48;
    maintLiabWeight: I80F48;
    initLiabWeight: I80F48;
    liquidationFee: I80F48;
    constructor(decoded: any);
    isEmpty(): boolean;
}
export declare class SpotMarketInfoLayout extends Structure {
    constructor(property: any);
    decode(b: any, offset: any): SpotMarketInfo;
    encode(src: any, b: any, offset: any): any;
}
export declare function spotMarketInfoLayout(property?: string): SpotMarketInfoLayout;
export declare class PerpMarketInfo {
    perpMarket: PublicKey;
    maintAssetWeight: I80F48;
    initAssetWeight: I80F48;
    maintLiabWeight: I80F48;
    initLiabWeight: I80F48;
    liquidationFee: I80F48;
    baseLotSize: BN;
    quoteLotSize: BN;
    constructor(decoded: any);
    isEmpty(): boolean;
}
export declare class PerpMarketInfoLayout extends Structure {
    constructor(property: any);
    decode(b: any, offset: any): PerpMarketInfo;
    encode(src: any, b: any, offset: any): any;
}
export declare function perpMarketInfoLayout(property?: string): PerpMarketInfoLayout;
export declare class PerpAccount {
    basePosition: BN;
    quotePosition: I80F48;
    longSettledFunding: I80F48;
    shortSettledFunding: I80F48;
    openOrders: PerpOpenOrders;
    constructor(decoded: any);
    getPnl(perpMarketInfo: PerpMarketInfo, price: I80F48): I80F48;
    simPositionHealth(perpMarketInfo: PerpMarketInfo, price: I80F48, assetWeight: I80F48, liabWeight: I80F48, baseChange: BN): I80F48;
    getHealth(perpMarketInfo: PerpMarketInfo, price: I80F48, assetWeight: I80F48, liabWeight: I80F48, longFunding: I80F48, shortFunding: I80F48): I80F48;
}
export declare class PerpAccountLayout extends Structure {
    constructor(property: any);
    decode(b: any, offset: any): PerpAccount;
    encode(src: any, b: any, offset: any): any;
}
export declare function perpAccountLayout(property?: string): PerpAccountLayout;
export declare class PerpOpenOrders {
    bidsQuantity: BN;
    asksQuantity: BN;
    isFreeBits: BN;
    isBidBits: BN;
    orders: BN[];
    clientOrderIds: BN[];
    constructor(decoded: any);
}
export declare class PerpOpenOrdersLayout extends Structure {
    constructor(property: any);
    decode(b: any, offset: any): PerpOpenOrders;
    encode(src: any, b: any, offset: any): any;
}
export declare function perpOpenOrdersLayout(property?: string): PerpOpenOrdersLayout;
export declare const MangoGroupLayout: any;
export declare const MangoAccountLayout: any;
export declare const RootBankLayout: any;
export declare const NodeBankLayout: any;
export declare const StubOracleLayout: any;
export declare const PerpMarketLayout: any;
export declare const PerpEventLayout: any;
export interface FillEvent {
    maker: boolean;
    owner: PublicKey;
    baseChange: BN;
    quoteChange: BN;
    longFunding: I80F48;
    shortFunding: I80F48;
}
export interface OutEvent {
    side: 'buy' | 'sell';
    slot: number;
    owner: PublicKey;
    quantity: BN;
}
export declare const PerpEventQueueLayout: any;
export declare class PerpEventQueue {
    head: BN;
    count: BN;
    seqNum: BN;
    events: any[];
    constructor(decoded: any);
    eventsSince(lastSeqNum: BN): {
        fill?: FillEvent;
        out?: OutEvent;
    }[];
}
export declare const BookSideLayout: any;
export declare class PriceCache {
    price: I80F48;
    lastUpdate: BN;
    isInitialized: boolean;
    constructor(decoded: any);
}
export declare class PriceCacheLayout extends Structure {
    constructor(property: any);
    decode(b: any, offset: any): PriceCache;
    encode(src: any, b: any, offset: any): any;
}
export declare function priceCacheLayout(property?: string): PriceCacheLayout;
export declare class RootBankCache {
    depositIndex: I80F48;
    borrowIndex: I80F48;
    lastUpdate: BN;
    constructor(decoded: any);
}
export declare class RootBankCacheLayout extends Structure {
    constructor(property: any);
    decode(b: any, offset: any): RootBankCache;
    encode(src: any, b: any, offset: any): any;
}
export declare function rootBankCacheLayout(property?: string): RootBankCacheLayout;
export declare class PerpMarketCache {
    longFunding: I80F48;
    shortFunding: I80F48;
    lastUpdate: BN;
    constructor(decoded: any);
}
export declare class PerpMarketCacheLayout extends Structure {
    constructor(property: any);
    decode(b: any, offset: any): PerpMarketCache;
    encode(src: any, b: any, offset: any): any;
}
export declare function perpMarketCacheLayout(property?: string): PerpMarketCacheLayout;
export declare const MangoCacheLayout: any;
export declare class MangoCache {
    publicKey: PublicKey;
    priceCache: PriceCache[];
    rootBankCache: RootBankCache[];
    perpMarketCache: PerpMarketCache[];
    constructor(publicKey: PublicKey, decoded: any);
}
export declare class NodeBank {
    publicKey: PublicKey;
    deposits: I80F48;
    borrows: I80F48;
    vault: PublicKey;
    constructor(publicKey: PublicKey, decoded: any);
}
export declare class EventQueueHeader {
    metaData: MetaData;
    head: number;
    count: number;
    seqNum: number;
    constructor(decoded: any);
}
export declare class EventQueueHeaderLayout extends Structure {
    constructor(property: any);
    decode(b: any, offset: any): EventQueueHeader;
    encode(src: any, b: any, offset: any): any;
}
export declare function eventQueueHeaderLayout(property?: string): EventQueueHeaderLayout;
export declare enum EventType {
    Fill = 0,
    Out = 1
}
export declare class AnyEvent {
    eventType: EventType;
}
export declare class AnyEventLayout extends Structure {
    constructor(property: any);
    decode(b: any, offset: any): EventQueueHeader;
    encode(src: any, b: any, offset: any): any;
}
export declare function anyEventLayout(property?: string): AnyEventLayout;
export declare class EventQueue {
    metaData: MetaData;
    head: number;
    count: number;
    seqNum: number;
    buf: AnyEvent[];
    constructor(decoded: any);
}
export declare class EventQueueLayout extends Structure {
    constructor(property: any);
    decode(b: any, offset: any): EventQueue;
    encode(src: any, b: any, offset: any): any;
}
export declare function eventQueueLayout(property?: string): EventQueueLayout;
export {};
//# sourceMappingURL=layout.d.ts.map