"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PriceCacheLayout = exports.PriceCache = exports.BookSideLayout = exports.PerpEventQueue = exports.PerpEventQueueLayout = exports.PerpEventLayout = exports.PerpMarketLayout = exports.StubOracleLayout = exports.NodeBankLayout = exports.RootBankLayout = exports.MangoAccountLayout = exports.MangoGroupLayout = exports.perpOpenOrdersLayout = exports.PerpOpenOrdersLayout = exports.PerpOpenOrders = exports.perpAccountLayout = exports.PerpAccountLayout = exports.PerpAccount = exports.perpMarketInfoLayout = exports.PerpMarketInfoLayout = exports.PerpMarketInfo = exports.spotMarketInfoLayout = exports.SpotMarketInfoLayout = exports.SpotMarketInfo = exports.tokenInfoLayout = exports.TokenInfoLayout = exports.TokenInfo = exports.metaDataLayout = exports.MetaDataLayout = exports.MetaData = exports.DataType = exports.publicKeyLayout = exports.PublicKeyLayout = exports.encodeMangoInstruction = exports.MangoInstructionLayout = exports.selfTradeBehaviorLayout = exports.orderTypeLayout = exports.sideLayout = exports.bool = exports.i128 = exports.u128 = exports.i64 = exports.u64 = exports.I80F48Layout = exports.OPTIMAL_RATE = exports.OPTIMAL_UTIL = exports.MAX_RATE = exports.MAX_NODE_BANKS = exports.MAX_PAIRS = exports.MAX_TOKENS = void 0;
exports.eventQueueLayout = exports.EventQueueLayout = exports.EventQueue = exports.anyEventLayout = exports.AnyEventLayout = exports.AnyEvent = exports.EventType = exports.eventQueueHeaderLayout = exports.EventQueueHeaderLayout = exports.EventQueueHeader = exports.NodeBank = exports.MangoCache = exports.MangoCacheLayout = exports.perpMarketCacheLayout = exports.PerpMarketCacheLayout = exports.PerpMarketCache = exports.rootBankCacheLayout = exports.RootBankCacheLayout = exports.RootBankCache = exports.priceCacheLayout = void 0;
const buffer_layout_1 = require("buffer-layout");
const web3_js_1 = require("@solana/web3.js");
const fixednum_1 = require("./fixednum");
const bn_js_1 = __importDefault(require("bn.js"));
const utils_1 = require("./utils");
exports.MAX_TOKENS = 32;
exports.MAX_PAIRS = exports.MAX_TOKENS - 1;
exports.MAX_NODE_BANKS = 8;
const MAX_BOOK_NODES = 1024;
exports.MAX_RATE = fixednum_1.I80F48.fromString('3.0');
exports.OPTIMAL_UTIL = fixednum_1.I80F48.fromString('0.7');
exports.OPTIMAL_RATE = fixednum_1.I80F48.fromString('0.2');
class _I80F48Layout extends buffer_layout_1.Blob {
    constructor(property) {
        super(16, property);
    }
    decode(b, offset) {
        let result = new bn_js_1.default(super.decode(b, offset), 10, 'le');
        result = result.fromTwos(8 * this['length']);
        return new fixednum_1.I80F48(result);
    }
    encode(src, b, offset) {
        src = src.toTwos(8 * this['length']);
        return super.encode(src.toArrayLike(Buffer, 'le', this['span']), b, offset);
    }
}
function I80F48Layout(property = '') {
    return new _I80F48Layout(property);
}
exports.I80F48Layout = I80F48Layout;
class BNLayout extends buffer_layout_1.Blob {
    constructor(number, property, signed = false) {
        super(number, property);
        this.signed = signed;
        // restore prototype chain
        Object.setPrototypeOf(this, new.target.prototype);
    }
    decode(b, offset) {
        let result = new bn_js_1.default(super.decode(b, offset), 10, 'le');
        if (this.signed)
            result = result.fromTwos(8 * this['length']);
        return result;
    }
    encode(src, b, offset) {
        if (this.signed)
            src = src.toTwos(8 * this['length']);
        return super.encode(src.toArrayLike(Buffer, 'le', this['span']), b, offset);
    }
}
function u64(property = '') {
    return new BNLayout(8, property);
}
exports.u64 = u64;
function i64(property = '') {
    return new BNLayout(8, property, true);
}
exports.i64 = i64;
function u128(property) {
    return new BNLayout(16, property);
}
exports.u128 = u128;
function i128(property) {
    return new BNLayout(16, property, true);
}
exports.i128 = i128;
class WrappedLayout extends buffer_layout_1.Layout {
    constructor(layout, decoder, encoder, property) {
        super(layout.span, property);
        this.layout = layout;
        this.decoder = decoder;
        this.encoder = encoder;
    }
    decode(b, offset) {
        return this.decoder(this.layout.decode(b, offset));
    }
    encode(src, b, offset) {
        return this.layout.encode(this.encoder(src), b, offset);
    }
    getSpan(b, offset) {
        return this.layout.getSpan(b, offset);
    }
}
function bool(property) {
    return new WrappedLayout(buffer_layout_1.u8(), decodeBool, encodeBool, property);
}
exports.bool = bool;
function decodeBool(value) {
    if (value === 0) {
        return false;
    }
    else if (value === 1) {
        return true;
    }
    throw new Error('Invalid bool: ' + value);
}
function encodeBool(value) {
    return value ? 1 : 0;
}
class EnumLayout extends buffer_layout_1.UInt {
    constructor(values, span, property) {
        super(span, property);
        this.values = values;
    }
    encode(src, b, offset) {
        if (this.values[src] !== undefined) {
            return super.encode(this.values[src], b, offset);
        }
        throw new Error('Invalid ' + this['property']);
    }
    decode(b, offset) {
        const decodedValue = super.decode(b, offset);
        const entry = Object.entries(this.values).find(([, value]) => value === decodedValue);
        if (entry) {
            return entry[0];
        }
        throw new Error('Invalid ' + this['property']);
    }
}
function sideLayout(property, span) {
    return new EnumLayout({ buy: 0, sell: 1 }, span, property);
}
exports.sideLayout = sideLayout;
function orderTypeLayout(property, span) {
    return new EnumLayout({ limit: 0, ioc: 1, postOnly: 2 }, span, property);
}
exports.orderTypeLayout = orderTypeLayout;
function selfTradeBehaviorLayout(property) {
    return new EnumLayout({ decrementTake: 0, cancelProvide: 1, abortTransaction: 2 }, 4, property);
}
exports.selfTradeBehaviorLayout = selfTradeBehaviorLayout;
/**
 * Need to implement layouts for each of the structs found in state.rs
 */
exports.MangoInstructionLayout = buffer_layout_1.union(buffer_layout_1.u32('instruction'));
exports.MangoInstructionLayout.addVariant(0, buffer_layout_1.struct([u64('signerNonce'), u64('validInterval')]), 'InitMangoGroup');
exports.MangoInstructionLayout.addVariant(1, buffer_layout_1.struct([]), 'InitMangoAccount');
exports.MangoInstructionLayout.addVariant(2, buffer_layout_1.struct([u64('quantity')]), 'Deposit');
exports.MangoInstructionLayout.addVariant(3, buffer_layout_1.struct([u64('quantity'), buffer_layout_1.u8('allowBorrow')]), 'Withdraw');
exports.MangoInstructionLayout.addVariant(4, buffer_layout_1.struct([u64('marketIndex'), u128('maintLeverage'), u128('initLeverage')]), 'AddSpotMarket');
exports.MangoInstructionLayout.addVariant(5, buffer_layout_1.struct([u64('marketIndex')]), 'AddToBasket');
exports.MangoInstructionLayout.addVariant(6, buffer_layout_1.struct([u64('quantity')]), 'Borrow');
exports.MangoInstructionLayout.addVariant(7, buffer_layout_1.struct([]), 'CachePrices');
exports.MangoInstructionLayout.addVariant(8, buffer_layout_1.struct([]), 'CacheRootBanks');
exports.MangoInstructionLayout.addVariant(9, buffer_layout_1.struct([
    sideLayout('side', 4),
    u64('limitPrice'),
    u64('maxBaseQuantity'),
    u64('maxQuoteQuantity'),
    selfTradeBehaviorLayout('selfTradeBehavior'),
    orderTypeLayout('orderType', 4),
    u64('clientId'),
    buffer_layout_1.u16('limit'),
]), 'PlaceSpotOrder');
exports.MangoInstructionLayout.addVariant(10, buffer_layout_1.struct([]), 'AddOracle');
exports.MangoInstructionLayout.addVariant(11, buffer_layout_1.struct([
    u64('marketIndex'),
    I80F48Layout('maintLeverage'),
    I80F48Layout('initLeverage'),
    i64('baseLotSize'),
    i64('quoteLotSize'),
]), 'AddPerpMarket');
exports.MangoInstructionLayout.addVariant(12, buffer_layout_1.struct([
    i64('price'),
    i64('quantity'),
    u64('clientOrderId'),
    sideLayout('side', 1),
    orderTypeLayout('orderType', 1),
]), 'PlacePerpOrder');
exports.MangoInstructionLayout.addVariant(13, buffer_layout_1.struct([u64('clientOrderId')]), 'CancelPerpOrderByClientId');
exports.MangoInstructionLayout.addVariant(14, buffer_layout_1.struct([i128('orderId'), sideLayout('side', 4)]), 'CancelPerpOrder');
exports.MangoInstructionLayout.addVariant(15, buffer_layout_1.struct([u64('limit')]), 'ConsumeEvents');
exports.MangoInstructionLayout.addVariant(16, buffer_layout_1.struct([]), 'CachePerpMarkets');
exports.MangoInstructionLayout.addVariant(17, buffer_layout_1.struct([]), 'UpdateFunding');
exports.MangoInstructionLayout.addVariant(18, buffer_layout_1.struct([I80F48Layout('price')]), 'SetOracle');
exports.MangoInstructionLayout.addVariant(19, buffer_layout_1.struct([]), 'SettleFunds');
exports.MangoInstructionLayout.addVariant(20, buffer_layout_1.struct([sideLayout('side', 4), u128('orderId')]), 'CancelSpotOrder');
exports.MangoInstructionLayout.addVariant(21, buffer_layout_1.struct([]), 'UpdateRootBank');
exports.MangoInstructionLayout.addVariant(22, buffer_layout_1.struct([u64('marketIndex')]), 'SettlePnl');
const instructionMaxSpan = Math.max(
// @ts-ignore
...Object.values(exports.MangoInstructionLayout.registry).map((r) => r.span));
function encodeMangoInstruction(data) {
    const b = Buffer.alloc(instructionMaxSpan);
    const span = exports.MangoInstructionLayout.encode(data, b);
    return b.slice(0, span);
}
exports.encodeMangoInstruction = encodeMangoInstruction;
class PublicKeyLayout extends buffer_layout_1.Blob {
    constructor(property) {
        super(32, property);
    }
    decode(b, offset) {
        return new web3_js_1.PublicKey(super.decode(b, offset));
    }
    encode(src, b, offset) {
        return super.encode(src.toBuffer(), b, offset);
    }
}
exports.PublicKeyLayout = PublicKeyLayout;
function publicKeyLayout(property = '') {
    return new PublicKeyLayout(property);
}
exports.publicKeyLayout = publicKeyLayout;
exports.DataType = {
    MangoGroup: 0,
    MangoAccount: 1,
    RootBank: 2,
    NodeBank: 3,
    PerpMarket: 4,
    Bids: 5,
    Asks: 6,
    MangoCache: 7,
    EventQueue: 8,
};
class MetaData {
    constructor(decoded) {
        Object.assign(this, decoded);
    }
}
exports.MetaData = MetaData;
class MetaDataLayout extends buffer_layout_1.Structure {
    constructor(property) {
        super([
            buffer_layout_1.u8('dataType'),
            buffer_layout_1.u8('version'),
            buffer_layout_1.u8('isInitialized'),
            buffer_layout_1.seq(buffer_layout_1.u8(), 5, 'padding'),
        ], property);
    }
    decode(b, offset) {
        return new MetaData(super.decode(b, offset));
    }
    encode(src, b, offset) {
        return super.encode(src.toBuffer(), b, offset);
    }
}
exports.MetaDataLayout = MetaDataLayout;
function metaDataLayout(property = '') {
    return new MetaDataLayout(property);
}
exports.metaDataLayout = metaDataLayout;
class TokenInfo {
    constructor(decoded) {
        Object.assign(this, decoded);
    }
    isEmpty() {
        return this.mint.equals(utils_1.zeroKey);
    }
}
exports.TokenInfo = TokenInfo;
class TokenInfoLayout extends buffer_layout_1.Structure {
    constructor(property) {
        super([
            publicKeyLayout('mint'),
            publicKeyLayout('rootBank'),
            buffer_layout_1.u8('decimals'),
            buffer_layout_1.seq(buffer_layout_1.u8(), 7, 'padding'),
        ], property);
    }
    decode(b, offset) {
        return new TokenInfo(super.decode(b, offset));
    }
    encode(src, b, offset) {
        return super.encode(src.toBuffer(), b, offset);
    }
}
exports.TokenInfoLayout = TokenInfoLayout;
function tokenInfoLayout(property = '') {
    return new TokenInfoLayout(property);
}
exports.tokenInfoLayout = tokenInfoLayout;
class SpotMarketInfo {
    constructor(decoded) {
        Object.assign(this, decoded);
    }
    isEmpty() {
        return this.spotMarket.equals(utils_1.zeroKey);
    }
}
exports.SpotMarketInfo = SpotMarketInfo;
class SpotMarketInfoLayout extends buffer_layout_1.Structure {
    constructor(property) {
        super([
            publicKeyLayout('spotMarket'),
            I80F48Layout('maintAssetWeight'),
            I80F48Layout('initAssetWeight'),
            I80F48Layout('maintLiabWeight'),
            I80F48Layout('initLiabWeight'),
            I80F48Layout('liquidationFee'),
        ], property);
    }
    decode(b, offset) {
        return new SpotMarketInfo(super.decode(b, offset));
    }
    encode(src, b, offset) {
        return super.encode(src.toBuffer(), b, offset);
    }
}
exports.SpotMarketInfoLayout = SpotMarketInfoLayout;
function spotMarketInfoLayout(property = '') {
    return new SpotMarketInfoLayout(property);
}
exports.spotMarketInfoLayout = spotMarketInfoLayout;
class PerpMarketInfo {
    constructor(decoded) {
        Object.assign(this, decoded);
    }
    isEmpty() {
        return this.perpMarket.equals(utils_1.zeroKey);
    }
}
exports.PerpMarketInfo = PerpMarketInfo;
class PerpMarketInfoLayout extends buffer_layout_1.Structure {
    constructor(property) {
        super([
            publicKeyLayout('perpMarket'),
            I80F48Layout('maintAssetWeight'),
            I80F48Layout('initAssetWeight'),
            I80F48Layout('maintLiabWeight'),
            I80F48Layout('initLiabWeight'),
            I80F48Layout('liquidationFee'),
            i64('baseLotSize'),
            i64('quoteLotSize'),
        ], property);
    }
    decode(b, offset) {
        return new PerpMarketInfo(super.decode(b, offset));
    }
    encode(src, b, offset) {
        return super.encode(src.toBuffer(), b, offset);
    }
}
exports.PerpMarketInfoLayout = PerpMarketInfoLayout;
function perpMarketInfoLayout(property = '') {
    return new PerpMarketInfoLayout(property);
}
exports.perpMarketInfoLayout = perpMarketInfoLayout;
class PerpAccount {
    constructor(decoded) {
        Object.assign(this, decoded);
    }
    getPnl(perpMarketInfo, price) {
        return fixednum_1.I80F48.fromI64(this.basePosition.mul(perpMarketInfo.baseLotSize))
            .mul(price)
            .add(this.quotePosition);
    }
    simPositionHealth(perpMarketInfo, price, assetWeight, liabWeight, baseChange) {
        const newBase = this.basePosition.add(baseChange);
        let health = this.quotePosition.sub(fixednum_1.I80F48.fromI64(baseChange.mul(perpMarketInfo.baseLotSize)).mul(price));
        if (newBase.gt(new bn_js_1.default(0))) {
            health = health.add(fixednum_1.I80F48.fromI64(newBase.mul(perpMarketInfo.baseLotSize))
                .mul(price)
                .mul(assetWeight));
        }
        else {
            health = health.add(fixednum_1.I80F48.fromI64(newBase.mul(perpMarketInfo.baseLotSize))
                .mul(price)
                .mul(liabWeight));
        }
        return health;
    }
    getHealth(perpMarketInfo, price, assetWeight, liabWeight, longFunding, shortFunding) {
        const bidsHealth = this.simPositionHealth(perpMarketInfo, price, assetWeight, liabWeight, this.openOrders.bidsQuantity);
        const asksHealth = this.simPositionHealth(perpMarketInfo, price, assetWeight, liabWeight, this.openOrders.asksQuantity.neg());
        const health = bidsHealth.lt(asksHealth) ? bidsHealth : asksHealth;
        if (this.basePosition.gt(new bn_js_1.default(0))) {
            return health.sub(longFunding
                .sub(this.longSettledFunding)
                .mul(fixednum_1.I80F48.fromI64(this.basePosition)));
        }
        else {
            return health.add(shortFunding
                .sub(this.shortSettledFunding)
                .mul(fixednum_1.I80F48.fromI64(this.basePosition)));
        }
    }
}
exports.PerpAccount = PerpAccount;
class PerpAccountLayout extends buffer_layout_1.Structure {
    constructor(property) {
        super([
            i64('basePosition'),
            I80F48Layout('quotePosition'),
            I80F48Layout('longSettledFunding'),
            I80F48Layout('shortSettledFunding'),
            perpOpenOrdersLayout('openOrders'),
        ], property);
    }
    decode(b, offset) {
        return new PerpAccount(super.decode(b, offset));
    }
    encode(src, b, offset) {
        return super.encode(src.toBuffer(), b, offset);
    }
}
exports.PerpAccountLayout = PerpAccountLayout;
function perpAccountLayout(property = '') {
    return new PerpAccountLayout(property);
}
exports.perpAccountLayout = perpAccountLayout;
class PerpOpenOrders {
    constructor(decoded) {
        Object.assign(this, decoded);
    }
}
exports.PerpOpenOrders = PerpOpenOrders;
class PerpOpenOrdersLayout extends buffer_layout_1.Structure {
    constructor(property) {
        super([
            i64('bidsQuantity'),
            i64('asksQuantity'),
            buffer_layout_1.u32('isFreeBits'),
            buffer_layout_1.u32('isBidBits'),
            buffer_layout_1.seq(i128(), exports.MAX_TOKENS, 'orders'),
            buffer_layout_1.seq(u64(), exports.MAX_TOKENS, 'clientOrderIds'),
        ], property);
    }
    decode(b, offset) {
        return new PerpOpenOrders(super.decode(b, offset));
    }
    encode(src, b, offset) {
        return super.encode(src.toBuffer(), b, offset);
    }
}
exports.PerpOpenOrdersLayout = PerpOpenOrdersLayout;
function perpOpenOrdersLayout(property = '') {
    return new PerpOpenOrdersLayout(property);
}
exports.perpOpenOrdersLayout = perpOpenOrdersLayout;
exports.MangoGroupLayout = buffer_layout_1.struct([
    metaDataLayout('metaData'),
    u64('numOracles'),
    buffer_layout_1.seq(tokenInfoLayout(), exports.MAX_TOKENS, 'tokens'),
    buffer_layout_1.seq(spotMarketInfoLayout(), exports.MAX_PAIRS, 'spotMarkets'),
    buffer_layout_1.seq(perpMarketInfoLayout(), exports.MAX_PAIRS, 'perpMarkets'),
    buffer_layout_1.seq(publicKeyLayout(), exports.MAX_PAIRS, 'oracles'),
    u64('signerNonce'),
    publicKeyLayout('signerKey'),
    publicKeyLayout('admin'),
    publicKeyLayout('dexProgramId'),
    publicKeyLayout('mangoCache'),
    u64('validInterval'),
]);
exports.MangoAccountLayout = buffer_layout_1.struct([
    metaDataLayout('metaData'),
    publicKeyLayout('mangoGroup'),
    publicKeyLayout('owner'),
    buffer_layout_1.seq(bool(), exports.MAX_PAIRS, 'inMarginBasket'),
    buffer_layout_1.u8('numInMarginBasket'),
    buffer_layout_1.seq(I80F48Layout(), exports.MAX_TOKENS, 'deposits'),
    buffer_layout_1.seq(I80F48Layout(), exports.MAX_TOKENS, 'borrows'),
    buffer_layout_1.seq(publicKeyLayout(), exports.MAX_PAIRS, 'spotOpenOrders'),
    buffer_layout_1.seq(perpAccountLayout(), exports.MAX_PAIRS, 'perpAccounts'),
    bool('beingLiquidated'),
    buffer_layout_1.seq(buffer_layout_1.u8(), 7, 'padding'),
]);
exports.RootBankLayout = buffer_layout_1.struct([
    metaDataLayout('metaData'),
    u64('numNodeBanks'),
    buffer_layout_1.seq(publicKeyLayout(), exports.MAX_NODE_BANKS, 'nodeBanks'),
    I80F48Layout('depositIndex'),
    I80F48Layout('borrowIndex'),
    u64('lastUpdated'),
]);
exports.NodeBankLayout = buffer_layout_1.struct([
    metaDataLayout('metaData'),
    I80F48Layout('deposits'),
    I80F48Layout('borrows'),
    publicKeyLayout('vault'),
]);
exports.StubOracleLayout = buffer_layout_1.struct([
    I80F48Layout('price'),
    u64('lastUpdate'),
]);
exports.PerpMarketLayout = buffer_layout_1.struct([
    metaDataLayout('metaData'),
    publicKeyLayout('mangoGroup'),
    publicKeyLayout('bids'),
    publicKeyLayout('asks'),
    publicKeyLayout('eventQueue'),
    I80F48Layout('longFunding'),
    I80F48Layout('shortFunding'),
    i64('openInterest'),
    i64('quoteLotSize'),
    publicKeyLayout('indexOracle'),
    u64('lastUpdated'),
    u64('seqNum'),
    i64('contractSize'),
]);
exports.PerpEventLayout = buffer_layout_1.union(buffer_layout_1.u8('eventType'), buffer_layout_1.blob(87), 'event');
exports.PerpEventLayout.addVariant(0, buffer_layout_1.struct([
    bool('maker'),
    buffer_layout_1.seq(buffer_layout_1.u8(), 6),
    publicKeyLayout('owner'),
    i64('baseChange'),
    i64('quoteChange'),
    I80F48Layout('longFunding'),
    I80F48Layout('shortFunding'),
]), 'fill');
exports.PerpEventLayout.addVariant(1, buffer_layout_1.struct([
    sideLayout('side', 1),
    buffer_layout_1.u8('slot'),
    buffer_layout_1.seq(buffer_layout_1.u8(), 5),
    publicKeyLayout('owner'),
    i64('quantity'),
]), 'out');
exports.PerpEventQueueLayout = buffer_layout_1.struct([
    metaDataLayout('metaData'),
    u64('head'),
    u64('count'),
    u64('seqNum'),
    buffer_layout_1.seq(exports.PerpEventLayout, buffer_layout_1.greedy(exports.PerpEventLayout.span), 'events'),
]);
class PerpEventQueue {
    constructor(decoded) {
        Object.assign(this, decoded);
    }
    eventsSince(lastSeqNum) {
        const modulo64Uint = new bn_js_1.default('10000000000000000', 'hex');
        let missedEvents = this.seqNum
            .add(modulo64Uint)
            .sub(lastSeqNum)
            .mod(modulo64Uint);
        /*
        console.log({
          last: lastSeqNum.toString(),
          now: this.seqNum.toString(),
          missed: missedEvents.toString(),
          mod: modulo64Uint.toString(),
        });
        */
        const bufferLength = new bn_js_1.default(this.events.length);
        if (missedEvents.gte(bufferLength)) {
            missedEvents = bufferLength.sub(new bn_js_1.default(1));
        }
        const endIndex = this.head.add(this.count).mod(bufferLength);
        const startIndex = endIndex
            .add(bufferLength)
            .sub(missedEvents)
            .mod(bufferLength);
        /*
        console.log({
          bufLength: bufferLength.toString(),
          missed: missedEvents.toString(),
          head: this.head.toString(),
          count: this.count.toString(),
          end: endIndex.toString(),
          start: startIndex.toString(),
        });
        */
        const results = [];
        let index = startIndex;
        while (!index.eq(endIndex)) {
            const event = this.events[index.toNumber()];
            if (event.fill || event.out)
                results.push(event);
            index = index.add(new bn_js_1.default(1)).mod(bufferLength);
        }
        return results;
    }
}
exports.PerpEventQueue = PerpEventQueue;
const BOOK_NODE_SIZE = 72;
const BOOK_NODE_LAYOUT = buffer_layout_1.union(buffer_layout_1.u32('tag'), buffer_layout_1.blob(BOOK_NODE_SIZE - 4), 'node');
BOOK_NODE_LAYOUT.addVariant(0, buffer_layout_1.struct([]), 'uninitialized');
BOOK_NODE_LAYOUT.addVariant(1, buffer_layout_1.struct([
    // Only the first prefixLen high-order bits of key are meaningful
    buffer_layout_1.u32('prefixLen'),
    u128('key'),
    buffer_layout_1.seq(buffer_layout_1.u32(), 2, 'children'),
]), 'innerNode');
BOOK_NODE_LAYOUT.addVariant(2, buffer_layout_1.struct([
    buffer_layout_1.u8('ownerSlot'),
    buffer_layout_1.blob(3),
    u128('key'),
    publicKeyLayout('owner'),
    u64('quantity'),
    u64('clientOrderId'),
]), 'leafNode');
BOOK_NODE_LAYOUT.addVariant(3, buffer_layout_1.struct([buffer_layout_1.u32('next')]), 'freeNode');
BOOK_NODE_LAYOUT.addVariant(4, buffer_layout_1.struct([]), 'lastFreeNode');
exports.BookSideLayout = buffer_layout_1.struct([
    metaDataLayout('metaData'),
    buffer_layout_1.nu64('bumpIndex'),
    buffer_layout_1.nu64('freeListLen'),
    buffer_layout_1.u32('freeListHead'),
    buffer_layout_1.u32('rootNode'),
    buffer_layout_1.nu64('leafCount'),
    buffer_layout_1.seq(BOOK_NODE_LAYOUT, MAX_BOOK_NODES, 'nodes'),
]);
class PriceCache {
    constructor(decoded) {
        Object.assign(this, decoded);
    }
}
exports.PriceCache = PriceCache;
class PriceCacheLayout extends buffer_layout_1.Structure {
    constructor(property) {
        super([I80F48Layout('price'), u64('lastUpdate')], property);
    }
    decode(b, offset) {
        return new PriceCache(super.decode(b, offset));
    }
    encode(src, b, offset) {
        return super.encode(src.toBuffer(), b, offset);
    }
}
exports.PriceCacheLayout = PriceCacheLayout;
function priceCacheLayout(property = '') {
    return new PriceCacheLayout(property);
}
exports.priceCacheLayout = priceCacheLayout;
class RootBankCache {
    constructor(decoded) {
        Object.assign(this, decoded);
    }
}
exports.RootBankCache = RootBankCache;
class RootBankCacheLayout extends buffer_layout_1.Structure {
    constructor(property) {
        super([
            I80F48Layout('depositIndex'),
            I80F48Layout('borrowIndex'),
            u64('lastUpdate'),
        ], property);
    }
    decode(b, offset) {
        return new RootBankCache(super.decode(b, offset));
    }
    encode(src, b, offset) {
        return super.encode(src.toBuffer(), b, offset);
    }
}
exports.RootBankCacheLayout = RootBankCacheLayout;
function rootBankCacheLayout(property = '') {
    return new RootBankCacheLayout(property);
}
exports.rootBankCacheLayout = rootBankCacheLayout;
class PerpMarketCache {
    constructor(decoded) {
        Object.assign(this, decoded);
    }
}
exports.PerpMarketCache = PerpMarketCache;
class PerpMarketCacheLayout extends buffer_layout_1.Structure {
    constructor(property) {
        super([
            I80F48Layout('longFunding'),
            I80F48Layout('shortFunding'),
            u64('lastUpdate'),
        ], property);
    }
    decode(b, offset) {
        return new PerpMarketCache(super.decode(b, offset));
    }
    encode(src, b, offset) {
        return super.encode(src.toBuffer(), b, offset);
    }
}
exports.PerpMarketCacheLayout = PerpMarketCacheLayout;
function perpMarketCacheLayout(property = '') {
    return new PerpMarketCacheLayout(property);
}
exports.perpMarketCacheLayout = perpMarketCacheLayout;
exports.MangoCacheLayout = buffer_layout_1.struct([
    metaDataLayout('metaData'),
    buffer_layout_1.seq(priceCacheLayout(), exports.MAX_PAIRS, 'priceCache'),
    buffer_layout_1.seq(rootBankCacheLayout(), exports.MAX_TOKENS, 'rootBankCache'),
    buffer_layout_1.seq(perpMarketCacheLayout(), exports.MAX_PAIRS, 'perpMarketCache'),
]);
class MangoCache {
    constructor(publicKey, decoded) {
        this.publicKey = publicKey;
        Object.assign(this, decoded);
    }
}
exports.MangoCache = MangoCache;
class NodeBank {
    constructor(publicKey, decoded) {
        this.publicKey = publicKey;
        Object.assign(this, decoded);
    }
}
exports.NodeBank = NodeBank;
class EventQueueHeader {
    constructor(decoded) {
        Object.assign(this, decoded);
    }
}
exports.EventQueueHeader = EventQueueHeader;
class EventQueueHeaderLayout extends buffer_layout_1.Structure {
    constructor(property) {
        super([metaDataLayout('metaData'), u64('head'), u64('count'), u64('seqNum')], property);
    }
    decode(b, offset) {
        return new EventQueueHeader(super.decode(b, offset));
    }
    encode(src, b, offset) {
        return super.encode(src.toBuffer(), b, offset);
    }
}
exports.EventQueueHeaderLayout = EventQueueHeaderLayout;
function eventQueueHeaderLayout(property = '') {
    return new EventQueueHeaderLayout(property);
}
exports.eventQueueHeaderLayout = eventQueueHeaderLayout;
var EventType;
(function (EventType) {
    EventType[EventType["Fill"] = 0] = "Fill";
    EventType[EventType["Out"] = 1] = "Out";
})(EventType = exports.EventType || (exports.EventType = {}));
class AnyEvent {
}
exports.AnyEvent = AnyEvent;
class AnyEventLayout extends buffer_layout_1.Structure {
    constructor(property) {
        super([buffer_layout_1.u8('eventType'), buffer_layout_1.seq(buffer_layout_1.u8(), 7, 'padding')], property);
    }
    decode(b, offset) {
        return new EventQueueHeader(super.decode(b, offset));
    }
    encode(src, b, offset) {
        return super.encode(src.toBuffer(), b, offset);
    }
}
exports.AnyEventLayout = AnyEventLayout;
function anyEventLayout(property = '') {
    return new AnyEventLayout(property);
}
exports.anyEventLayout = anyEventLayout;
class EventQueue {
    constructor(decoded) {
        Object.assign(this, decoded);
    }
}
exports.EventQueue = EventQueue;
class EventQueueLayout extends buffer_layout_1.Structure {
    constructor(property) {
        //const headerLayout = eventQueueHeaderLayout('header');
        const queueLength = u64('count');
        console.log(queueLength);
        super([
            metaDataLayout('metaData'),
            u64('head'),
            queueLength,
            u64('seqNum'),
            buffer_layout_1.seq(anyEventLayout(), buffer_layout_1.offset(queueLength, -1), 'buf'),
        ], property);
    }
    decode(b, offset) {
        return new EventQueue(super.decode(b, offset));
    }
    encode(src, b, offset) {
        return super.encode(src.toBuffer(), b, offset);
    }
}
exports.EventQueueLayout = EventQueueLayout;
function eventQueueLayout(property = '') {
    return new EventQueueLayout(property);
}
exports.eventQueueLayout = eventQueueLayout;
//# sourceMappingURL=layout.js.map