"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable @typescript-eslint/no-non-null-assertion */
const os = __importStar(require("os"));
const fs = __importStar(require("fs"));
const client_1 = require("./client");
const web3_js_1 = require("@solana/web3.js");
const MangoGroup_1 = require("../src/MangoGroup");
const utils_1 = require("./utils");
const fixednum_1 = require("./fixednum");
const serum_1 = require("@project-serum/serum");
const Test = __importStar(require("../test/utils"));
const spl_token_1 = require("@solana/spl-token");
const token_1 = require("./token");
function assertEq(msg, a, b) {
    if (a !== b) {
        throw new Error(`${msg}: ${a} !== ${b}`);
    }
}
const mangoProgramId = new web3_js_1.PublicKey('32WeJ46tuY6QEkgydqzHYU5j85UT9m1cPJwFxPjuSVCt');
const serumDexPk = new web3_js_1.PublicKey('DESVgJVGajEgKGXhb6XmqDHGz3VjdgP7rEVESBgxmroY');
const quoteMintKey = new web3_js_1.PublicKey('EMjjdsqERN4wJUR9jMBax2pzqQPeGLNn5NeucbHpDUZK');
const btcMint = new web3_js_1.PublicKey('bypQzRBaSDWiKhoAw3hNkf35eF3z3AZCU8Sxks6mTPP');
const btcFaucetPk = new web3_js_1.PublicKey('454w2aqqmu3tzY3dgCh8gCk6jwQcxdo6ojvqj2JcLJqh');
const btcUsdSpotMarket = new web3_js_1.PublicKey('E1mfsnnCcL24JcDQxr7F2BpWjkyy5x2WHys8EL2pnCj9');
const connection = new web3_js_1.Connection('https://api.devnet.solana.com', 'processed');
const SLEEP_TIME = 2000;
const payer = new web3_js_1.Account(JSON.parse(fs.readFileSync(os.homedir() + '/.config/solana/devnet.json', 'utf-8')));
const client = new client_1.MangoClient(connection, mangoProgramId);
function init_mango_group_and_spot_market() {
    return __awaiter(this, void 0, void 0, function* () {
        const groupKey = yield client.initMangoGroup(quoteMintKey, serumDexPk, 500, payer);
        console.log('= mango group created =');
        let mangoGroup = yield client.getMangoGroup(groupKey);
        assertEq('quoteMint', mangoGroup.tokens[MangoGroup_1.QUOTE_INDEX].mint.toBase58(), quoteMintKey.toBase58());
        assertEq('admin', mangoGroup.admin.toBase58(), payer.publicKey.toBase58());
        assertEq('serumDexPk', mangoGroup.dexProgramId.toBase58(), serumDexPk.toBase58());
        yield utils_1.sleep(SLEEP_TIME);
        let btcOraclePk;
        try {
            console.log('= adding oracle =');
            btcOraclePk = yield Test.createOracle(connection, mangoProgramId, payer);
            yield client.addOracle(mangoGroup, btcOraclePk, payer);
            yield client.setOracle(mangoGroup, btcOraclePk, payer, fixednum_1.I80F48.fromNumber(40000));
        }
        catch (err) {
            console.log('Error on adding oracle', `${err}`);
        }
        yield utils_1.sleep(SLEEP_TIME);
        const initLeverage = 5;
        const maintLeverage = initLeverage * 2;
        const marketIndex = 0;
        console.log('= adding spot market =');
        yield client.addSpotMarket(mangoGroup, btcUsdSpotMarket, btcMint, payer, marketIndex, maintLeverage, initLeverage);
        yield utils_1.sleep(SLEEP_TIME);
        mangoGroup = yield client.getMangoGroup(groupKey);
        return mangoGroup;
    });
}
function test_place_spot_order() {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function* () {
        console.log('= starting =');
        let mangoGroup = yield init_mango_group_and_spot_market();
        const userQuoteTokenAcc = yield token_1.findLargestTokenAccountForOwner(connection, payer.publicKey, quoteMintKey);
        const userBtcTokenAcc = yield token_1.findLargestTokenAccountForOwner(connection, payer.publicKey, btcMint);
        const mangoAccountPk = yield client.initMangoAccount(mangoGroup, payer);
        yield utils_1.sleep(SLEEP_TIME); // devnet rate limits
        let mangoAccount = yield client.getMangoAccount(mangoAccountPk, serumDexPk);
        yield utils_1.sleep(SLEEP_TIME); // devnet rate limits
        let rootBanks = yield mangoGroup.loadRootBanks(client.connection);
        const usdcRootBank = rootBanks[MangoGroup_1.QUOTE_INDEX];
        if (!usdcRootBank)
            throw new Error('no root bank');
        const quoteNodeBank = usdcRootBank.nodeBankAccounts[0];
        const marketIndex = 0;
        // run keeper fns
        yield client.cacheRootBanks(mangoGroup.publicKey, mangoGroup.mangoCache, [
            mangoGroup.tokens[marketIndex].rootBank,
            mangoGroup.tokens[MangoGroup_1.QUOTE_INDEX].rootBank,
        ], payer);
        yield utils_1.sleep(SLEEP_TIME); // devnet rate limits
        try {
            console.log('= depositing =');
            yield client.deposit(mangoGroup, mangoAccount, payer, mangoGroup.tokens[MangoGroup_1.QUOTE_INDEX].rootBank, (_a = usdcRootBank.nodeBanks) === null || _a === void 0 ? void 0 : _a[0], quoteNodeBank.vault, userQuoteTokenAcc.publicKey, 1000);
        }
        catch (err) {
            console.log('Error on deposit', `${err}`);
        }
        yield utils_1.sleep(SLEEP_TIME); // avoid devnet rate limit
        console.log('= adding to basket =');
        // await client.addToBasket(mangoGroup, mangoAccount, payer, marketIndex);
        mangoGroup = yield client.getMangoGroup(mangoGroup.publicKey);
        yield utils_1.sleep(SLEEP_TIME); // avoid devnet rate limit
        // run keeper fns
        const cacheRootBanksTxID = yield client.cacheRootBanks(mangoGroup.publicKey, mangoGroup.mangoCache, [
            mangoGroup.tokens[marketIndex].rootBank,
            mangoGroup.tokens[MangoGroup_1.QUOTE_INDEX].rootBank,
        ], payer);
        yield client.cachePrices(mangoGroup.publicKey, mangoGroup.mangoCache, [mangoGroup.oracles[marketIndex]], payer);
        console.log('= cache updated =', cacheRootBanksTxID);
        rootBanks = yield mangoGroup.loadRootBanks(client.connection);
        yield utils_1.sleep(SLEEP_TIME); // avoid devnet rate limit
        const btcRootBank = rootBanks[marketIndex];
        if (!btcRootBank)
            throw new Error('no root bank');
        const btcNodeBanks = yield btcRootBank.loadNodeBanks(client.connection);
        const filteredBtcNodeBanks = btcNodeBanks.filter((nodeBank) => !!nodeBank);
        if (!filteredBtcNodeBanks[0])
            throw new Error('node banks empty');
        console.log('= airdropping in btc vault =');
        const multiplier = Math.pow(10, 6);
        const btcAmount = 5 * multiplier;
        yield Test.airdropTokens(connection, payer, btcFaucetPk, filteredBtcNodeBanks[0].vault, btcMint, new spl_token_1.u64(btcAmount));
        utils_1.sleep(SLEEP_TIME);
        yield client.updateRootBank(mangoGroup.publicKey, btcRootBank.publicKey, filteredBtcNodeBanks.map((bank) => bank.publicKey), payer);
        yield client.updateRootBank(mangoGroup.publicKey, usdcRootBank.publicKey, [quoteNodeBank.publicKey], payer);
        yield utils_1.sleep(SLEEP_TIME); // devnet rate limits
        mangoAccount = yield client.getMangoAccount(mangoAccountPk, serumDexPk);
        const btcSpotMarket = yield serum_1.Market.load(connection, btcUsdSpotMarket, {}, serumDexPk);
        try {
            console.log('= placing spot order =');
            yield client.placeSpotOrder(mangoGroup, mangoAccount, mangoGroup.mangoCache, btcSpotMarket, payer, 'buy', 30000, // price
            0.0001, // size
            'limit');
        }
        catch (e) {
            console.log('Error placing order', `${e}`);
        }
        yield utils_1.sleep(SLEEP_TIME);
        mangoAccount = yield client.getMangoAccount(mangoAccountPk, serumDexPk);
        console.log('= borrow and withdraw =');
        yield client.withdraw(mangoGroup, mangoAccount, payer, mangoGroup.tokens[marketIndex].rootBank, (_b = btcRootBank.nodeBanks) === null || _b === void 0 ? void 0 : _b[0], filteredBtcNodeBanks[0].vault, userBtcTokenAcc.publicKey, 0.5, // withdraw amount
        true);
    });
}
test_place_spot_order();
//# sourceMappingURL=test.js.map