"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createTokenAccountInstrs = exports.createTokenAccount = exports.buildAirdropTokensIx = exports.airdropTokens = exports.createTokenAccountWithBalance = exports.createAccount = exports.createOracle = exports.airdropSol = exports.createDevnetConnection = exports._sendTransaction = exports.USDCMint = exports.DexProgramId = exports.MangoProgramId = void 0;
const serum_1 = require("@project-serum/serum");
const spl_token_1 = require("@solana/spl-token");
const web3_js_1 = require("@solana/web3.js");
const layout_1 = require("../src/layout");
const utils_1 = require("../src/utils");
exports.MangoProgramId = new web3_js_1.PublicKey('Hc12EyQQ3XVNEE5URg7XjjtZA8sbUPnMeT1CXGbwN6ei');
exports.DexProgramId = new web3_js_1.PublicKey('DESVgJVGajEgKGXhb6XmqDHGz3VjdgP7rEVESBgxmroY');
exports.USDCMint = new web3_js_1.PublicKey('H6hy7Ykzc43EuGivv7VVuUKNpKgUoFAfUY3wdPr4UyRX');
const FAUCET_PROGRAM_ID = new web3_js_1.PublicKey('4bXpkKSV8swHSnwqtzuboGPaPDeEgAn4Vt8GfarV5rZt');
const getPDA = () => {
    return web3_js_1.PublicKey.findProgramAddress([Buffer.from('faucet')], FAUCET_PROGRAM_ID);
};
function _sendTransaction(connection, transaction, signers) {
    return __awaiter(this, void 0, void 0, function* () {
        yield utils_1.sleep(1000);
        const signature = yield connection.sendTransaction(transaction, signers);
        try {
            yield connection.confirmTransaction(signature);
        }
        catch (e) {
            console.info('Error while confirming, trying again');
            yield connection.confirmTransaction(signature);
        }
        return signature;
    });
}
exports._sendTransaction = _sendTransaction;
function createDevnetConnection() {
    return new web3_js_1.Connection('https://api.devnet.solana.com', 'processed');
}
exports.createDevnetConnection = createDevnetConnection;
function airdropSol(connection, account, amount) {
    return __awaiter(this, void 0, void 0, function* () {
        const roundedSolAmount = Math.round(amount);
        console.info(`Requesting ${roundedSolAmount} SOL`);
        const generousAccount = [
            115, 98, 128, 18, 66, 112, 147, 244, 46, 244, 118, 106, 91, 202, 56, 83, 58,
            71, 89, 226, 32, 177, 177, 240, 189, 23, 209, 176, 138, 119, 130, 140, 6,
            149, 55, 70, 215, 34, 108, 133, 225, 117, 38, 141, 74, 246, 232, 76, 176,
            10, 207, 221, 68, 179, 115, 158, 106, 133, 35, 30, 4, 177, 124, 5,
        ];
        const backupAcc = new web3_js_1.Account(generousAccount);
        const tx = new web3_js_1.Transaction();
        tx.add(web3_js_1.SystemProgram.transfer({
            fromPubkey: backupAcc.publicKey,
            lamports: roundedSolAmount * 1e9,
            toPubkey: account.publicKey,
        }));
        const signers = [backupAcc];
        const signerPks = signers.map((x) => x.publicKey);
        tx.setSigners(...signerPks);
        yield _sendTransaction(connection, tx, signers);
    });
}
exports.airdropSol = airdropSol;
function createOracle(connection, programId, payer) {
    return __awaiter(this, void 0, void 0, function* () {
        const createOracleIns = yield utils_1.createAccountInstruction(connection, payer.publicKey, layout_1.StubOracleLayout.span, programId);
        const tx = new web3_js_1.Transaction();
        tx.add(createOracleIns.instruction);
        const signers = [payer, createOracleIns.account];
        const signerPks = signers.map((x) => x.publicKey);
        tx.setSigners(...signerPks);
        yield _sendTransaction(connection, tx, signers);
        return createOracleIns.account.publicKey;
    });
}
exports.createOracle = createOracle;
function createAccount(connection, solBalance = 5) {
    return __awaiter(this, void 0, void 0, function* () {
        const account = new web3_js_1.Account();
        if (solBalance >= 1) {
            yield airdropSol(connection, account, solBalance);
        }
        return account;
    });
}
exports.createAccount = createAccount;
function createTokenAccountWithBalance(connection, owner, tokenMint, tokenDecimals, faucetId, amount) {
    return __awaiter(this, void 0, void 0, function* () {
        const multiplier = Math.pow(10, tokenDecimals);
        const processedAmount = amount * multiplier;
        let ownedTokenAccountPk = null;
        ownedTokenAccountPk = yield createTokenAccount(connection, tokenMint, owner);
        if (amount > 0) {
            yield airdropTokens(connection, owner, faucetId, ownedTokenAccountPk, tokenMint, new spl_token_1.u64(processedAmount));
        }
        return ownedTokenAccountPk;
    });
}
exports.createTokenAccountWithBalance = createTokenAccountWithBalance;
function airdropTokens(connection, feePayerAccount, faucetPubkey, tokenDestinationPublicKey, mint, amount) {
    return __awaiter(this, void 0, void 0, function* () {
        const ix = yield buildAirdropTokensIx(amount, mint, tokenDestinationPublicKey, faucetPubkey);
        const tx = new web3_js_1.Transaction();
        tx.add(ix);
        const signers = [feePayerAccount];
        yield _sendTransaction(connection, tx, signers);
        return tokenDestinationPublicKey.toBase58();
    });
}
exports.airdropTokens = airdropTokens;
function buildAirdropTokensIx(amount, tokenMintPublicKey, destinationAccountPubkey, faucetPubkey) {
    return __awaiter(this, void 0, void 0, function* () {
        const pubkeyNonce = yield getPDA();
        const keys = [
            { pubkey: pubkeyNonce[0], isSigner: false, isWritable: false },
            { pubkey: tokenMintPublicKey, isSigner: false, isWritable: true },
            { pubkey: destinationAccountPubkey, isSigner: false, isWritable: true },
            {
                pubkey: serum_1.TokenInstructions.TOKEN_PROGRAM_ID,
                isSigner: false,
                isWritable: false,
            },
            { pubkey: faucetPubkey, isSigner: false, isWritable: false },
        ];
        return new web3_js_1.TransactionInstruction({
            programId: FAUCET_PROGRAM_ID,
            data: Buffer.from([1, ...amount.toArray('le', 8)]),
            keys,
        });
    });
}
exports.buildAirdropTokensIx = buildAirdropTokensIx;
function createTokenAccount(connection, mint, owner) {
    return __awaiter(this, void 0, void 0, function* () {
        const newAccount = new web3_js_1.Account();
        const tx = new web3_js_1.Transaction();
        const signers = [owner, newAccount];
        const signerPks = signers.map((x) => x.publicKey);
        tx.add(...(yield createTokenAccountInstrs(connection, newAccount.publicKey, mint, owner.publicKey)));
        tx.setSigners(...signerPks);
        yield _sendTransaction(connection, tx, signers);
        return newAccount.publicKey;
    });
}
exports.createTokenAccount = createTokenAccount;
function createTokenAccountInstrs(connection, newAccountPubkey, mint, ownerPk, lamports) {
    return __awaiter(this, void 0, void 0, function* () {
        if (lamports === undefined)
            lamports = yield connection.getMinimumBalanceForRentExemption(165);
        return [
            web3_js_1.SystemProgram.createAccount({
                fromPubkey: ownerPk,
                newAccountPubkey,
                space: 165,
                lamports,
                programId: serum_1.TokenInstructions.TOKEN_PROGRAM_ID,
            }),
            serum_1.TokenInstructions.initializeAccount({
                account: newAccountPubkey,
                mint,
                owner: ownerPk,
            }),
        ];
    });
}
exports.createTokenAccountInstrs = createTokenAccountInstrs;
//# sourceMappingURL=utils.js.map